from pathlib import Path
from typing import Any

from jinja2 import Environment
from jinja2 import FileSystemLoader

from oas_client.types import resolve_type


def render_client(spec: dict[str, Any], template_dir: Path) -> str:
    env = Environment(loader=FileSystemLoader(template_dir))
    template = env.get_template("client.jinja2")

    paths: dict[str, dict[str, dict[str, Any]]] = spec.get("paths", {})
    functions: list[dict[str, Any]] = []

    for path, methods in paths.items():
        for method, op in methods.items():
            op_id = op["operationId"]

            # Extract response schemas
            schemas: list[str] = []
            responses: dict[str, dict[str, Any]] = op.get("responses", {})
            for _, res in responses.items():
                if "application/json" in res.get("content", {}):
                    schema_ref = res["content"]["application/json"]["schema"][
                        "$ref"
                    ]
                    schemas.append("responses." + schema_ref.split("/")[-1])

            # Extract request body schema
            body = None
            if "requestBody" in op:
                content = op["requestBody"].get("content", {})
                if "application/json" in content:
                    schema_ref = content["application/json"]["schema"]["$ref"]
                    body = "requests." + schema_ref.split("/")[-1]

            # Extract query/path/header parameters
            parameters: list[dict[str, str]] = []
            queries: list[dict[str, str]] = []
            for param in op.get("parameters", []):
                name = param["name"]
                schema = param.get("schema", {})
                typ, _ = resolve_type(schema)
                if param["in"] == "path":
                    parameters.append({"name": name, "type": typ})
                if param["in"] == "query":
                    queries.append({"name": name, "type": typ})

            functions.append(
                {
                    "func_name": op_id,
                    "url": path,
                    "http_method": method,
                    "return": " | ".join(schemas),
                    "body": body,
                    "parameters": parameters,
                    "queries": queries,
                }
            )
    # NOTE: typed parameter and queries are not implemented yet
    return template.render(functions=functions)



from typing import Any


def resolve_type(prop: dict[str, Any]) -> tuple[str, list[tuple[str, str]]]:
    """
    Returns type of the property and additional imports required
    """
    if "$ref" in prop:
        schema = prop["$ref"].split("/")[-1]
        return f'"{schema}"', []

    any_of: list[dict[str, Any]] | None = prop.get("anyOf")
    if any_of is not None:
        types: list[str] = []
        imports: list[tuple[str, str]] = []
        for p in any_of:
            t, i = resolve_type(p)
            types.append(t)
            imports += i
        return " | ".join(types), imports
    t = prop.get("type")
    if t == "string":
        return "str", []
    elif t == "integer":
        return "int", []
    elif t == "number":
        return "float", []
    elif t == "boolean":
        return "bool", []
    elif t == "null":
        return "None", []
    elif t == "array":
        item_type, imports = resolve_type(prop.get("items", {}))
        return f"list[{item_type}]", imports
    elif t == "object":
        return "dict[str, Any]", [("typing", "Any")]
    return "Any", [("typing", "Any")]



import argparse
import json
import os
from pathlib import Path

from oas_client.client import render_client
from oas_client.schemas import render_schemas

BASE_DIR = Path(__file__).parent

import subprocess
from pathlib import Path


def main():
    parser = argparse.ArgumentParser(
        description="Generate OpenAPI client from an OpenAPI JSON spec."
    )
    parser.add_argument("openapi_json", help="Path to the OpenAPI JSON file.")
    parser.add_argument(
        "--output-dir", help="Path to the output directory.", default="client"
    )
    parser.add_argument(
        "--template-dir",
        help="Path to the template directory.",
        default=BASE_DIR / "templates",
    )
    parser.add_argument(
        "--no-formatting",
        help="Disables code formatting (black)",
        action="store_true",
    )
    parser.add_argument(
        "--no-import-sorting",
        help="Disables import sorting (isort)",
        action="store_true",
    )
    args = parser.parse_args()

    openapi_json = Path(args.openapi_json)
    output_dir = Path(args.output_dir)
    template_dir = Path(args.template_dir)

    with open(openapi_json) as f:
        spec = json.load(f)

    os.makedirs(output_dir, exist_ok=True)
    (output_dir / "__init__.py").write_text("")
    (output_dir / "responses.py").write_text(
        render_schemas(spec, template_dir)
    )
    (output_dir / "requests.py").write_text(
        render_schemas(spec, template_dir, partial=True)
    )
    (output_dir / "client.py").write_text(render_client(spec, template_dir))

    if not args.no_import_sorting:
        try:
            subprocess.run(["isort", str(output_dir)], check=True)
        except FileNotFoundError:
            print("isort not found in path. Skipping...")

    if not args.no_formatting:
        try:
            subprocess.run(["black", str(output_dir)], check=True)
        except FileNotFoundError:
            print("black not found in path. Skipping...")


if __name__ == "__main__":
    main()



from pathlib import Path
from typing import Any

from jinja2 import Environment
from jinja2 import FileSystemLoader

from oas_client.types import resolve_type


def find_schemas(
    spec: dict[str, Any], partial: bool = False
) -> tuple[list[dict[str, Any]], set[tuple[str, str]]]:

    schemas: dict[str, dict[str, Any]] = spec.get("components", {}).get(
        "schemas", {}
    )
    output: list[dict[str, Any]] = []
    imports: set[tuple[str, str]] = set()
    for name, schema in schemas.items():
        imports.add(("typing", "TypedDict"))
        required: set[str] = set(schema.get("required", []))
        props: dict[str, dict[str, Any]] = schema.get("properties", {})

        fields: list[dict[str, str]] = []
        for prop_name, prop in props.items():
            type_str, imps = resolve_type(prop)
            if partial and prop_name not in required:
                imports.add(("typing", "NotRequired"))
                type_str = f"NotRequired[{type_str}]"
            fields.append({"name": prop_name, "type": type_str})
            for i in imps:
                imports.add(i)
        output.append({"name": name, "fields": fields})
    return output, imports


def render_schemas(
    spec: dict[str, Any], template_dir: Path, partial: bool = False
) -> str:
    """
    When partial is set to true NotRequired is added to type when applicable,
    NotRequired is only necessary for request body not responses
    """
    schemas, imports = find_schemas(spec, partial)
    env = Environment(loader=FileSystemLoader(template_dir))
    template = env.get_template("schemas.jinja2")
    output_code = template.render(schemas=schemas, imports=imports)
    return output_code
